#include "ParseError.h"
#include "Info.h"
#include "Printer.h"
#include <sstream>
using namespace std;

RedefinedError::RedefinedError(PIdent *ident, VarInfoPtr orig) {
	stringstream ss;
	ss << "Variable \"" << ident->string_ << "\" at line " << ident->lineno << " redeclared! First declaration at line " << orig->lineLocation;
	msg = ss.str();
	line = ident->lineno;
}

UndefinedError::UndefinedError(PIdent *ident) {
	stringstream ss;
	ss << "Undefined reference to \"" << ident->string_ << "\" at line " << ident->lineno;
	msg = ss.str();
	line = ident->lineno;
}


InvalidTypeError::InvalidTypeError(Type &expected, vector<shared_ptr<Type>> received, Visitable *expr) {
	stringstream ss;
	ss << "Invalid expression type at line " << expr->lineno << ". Expected \"" << expected.printName() << "\", instead received ";
	bool fst = true;
	for (auto i : received) {
		if (fst) fst = false;
		else ss << ", ";
		ss << i->printName();
	}
	ss << ".";
	if (expr) {
		ss << " In: ";
		PrintAbsyn p;
		ss << p.print(expr);
	}
	msg = ss.str();
	line = expr->lineno;

}